/*
This file is part of django-ca (https://github.com/mathiasertl/django-ca).

django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with django-ca.  If not,
see <http://www.gnu.org/licenses/>.
*/

/* Viewing the CSR and PEM as code. */
fieldset.as-code .field-csr p,
fieldset.as-code .field-pub p,
.shell
{
    white-space: pre;
    white-space: pre-wrap;
    font-family: monospace;
    background: #faf8f0;
    border: 1px solid #bebab0;
    padding: 10px 15px !important;
    display: table;
    margin-top: 3px;
}
